# @Time: 2024年10月03日 03:04
# @Author: orcakill
# @File: opencv.py
# @Description: opencv相关算法api
import cv2
import imageio
import numpy as np


class BasicOpenCv:

    @staticmethod
    def draw_rectangle(screen, x1: int, y1: int, x2: int, y2: int, img_path: str):
        """
        画图，根据指定范围的坐标在原图上画框，并打印到指定地址

        :param screen: 图像ndarray
        :param x1: 图片左上角横坐标
        :param y1: 图片左上角纵坐标
        :param x2: 图片右下角横坐标
        :param y2: 图片右下角纵坐标
        :param img_path: 图片路径
        :return: None
        """

        cv2.rectangle(screen, (x1, y1), (x2, y2), (0, 0, 255), 2)
        # 保存图片到本地磁盘
        imageio.imsave(img_path, screen)

    @staticmethod
    def draw_point(screen, pos, img_path: str):
        """
        画图，根据指定范围的坐标在原图上画框，并打印到指定地址

        :param screen: 图像 ndarray
        :param pos: (横坐标,纵坐标）
        :param img_path: 图片路径
        :return: None
        """
        rgb_image = cv2.cvtColor(screen, cv2.COLOR_RGB2BGR)
        print(pos)
        cv2.circle(rgb_image, pos, 5, (255, 0, 0), -1)
        # 保存图片到本地磁盘
        imageio.imsave(img_path, rgb_image)

    @staticmethod
    def get_color_format(screen):
        """
        判断数组的颜色空间格式 

        :param screen: 图像数组 ndarray
        :return: str
        """

        r_mean = np.mean(screen[:, :, 0])
        g_mean = np.mean(screen[:, :, 1])
        b_mean = np.mean(screen[:, :, 2])

        if r_mean > g_mean and r_mean > b_mean:
            return 'RGB'
        elif b_mean > r_mean and b_mean > g_mean:
            return 'BGR'
        else:
            # 可能需要更复杂的判断逻辑或者无法确定
            return False



# _*_ coding:UTF-8 _*_
# @Time: 2024年10月02日 20:25
# @Author: orcakill
# @File: utils_encoding.py
# @Description: 文件格式工具类

import chardet


class UtilsEncoding:

    @staticmethod
    def check_file_encoding(file_path: str):
        """
        检查文件的编码格式并输出

        :param file_path: 文件路径
        :return: None
        """
        with open(file_path, 'rb') as f:
            result = chardet.detect(f.read())
            encoding = result['encoding']
            print("Detected encoding:", encoding)

    @staticmethod
    def check_file_content(file_path: str):
        """
        检查文档的格式

        :param file_path:
        :return:
        """
        install_requires=[]
        with open(file_path, encoding="UTF-16") as file:
            for line in file:
                install_requires.append(line.strip())
        print(install_requires)


if __name__ == '__main__':
    UtilsEncoding.check_file_encoding("utils_version.py")



# _*_ coding:UTF-8 _*_
# @Time: 2024年10月02日 22:49
# @Author: orcakill
# @File: version.py.py
# @Description: 版本号
__version__ = "1.0.1"

import os
import sys


def get_autopc_version():
    """
    当前版本设置

    :return:
    """
    pip_pkg_dir = os.path.join(os.path.dirname(__file__), "..", "..")
    pip_pkg_dir = os.path.abspath(pip_pkg_dir)

    return (
        'autopc {} from {} (python {})'.format(
            __version__, pip_pkg_dir, sys.version[:3],
        )
    )


def show_version():
    sys.stdout.write(get_autopc_version())
    sys.stdout.write(os.linesep)
    sys.exit()



# @Time: 2024年10月01日 03:23
# @Author: orcakill
# @File: airtest.py
# @Description: airtest相关的api接口
import logging
import random

from airtest import aircv
from airtest.aircv import cv2_2_pil
from airtest.core.android import Android
from airtest.core.android.cap_methods.screen_proxy import ScreenProxy
from airtest.core.api import *
from airtest.core.helper import G
from airtest.core.settings import Settings

#  控制airtest的日志输出
log_airtest = logging.getLogger("airtest")
log_airtest.setLevel(logging.CRITICAL)
# ·图片点击识别等待时间(秒）·
WAIT_TIME = 0
# 图像识别阈值
THRESHOLD = 0.7
#  长按时间
DURATION = 0.1


class BasicAirtest:
    """
    airtest的基础api
    """

    @staticmethod
    def auto_setup(connect_name: str, device_type: str = 'Android', ip: str = '127.0.0.1:5037', hwnd: str = '',
                   title: str = ''):
        """
        设备连接，支持Android和Windows，默认使用Android连接

        :param connect_name: 连接名称
        :param device_type: 设备类型
        :param ip: IP和端口（Android连接）
        :param hwnd: 句柄（windows连接）
        :param title: 窗口标题（windows连接）
        :return: None
        """
        # 根据设备类型选择连接方式
        devices = ''
        if device_type == 'Android':
            devices = 'Android://' + ip + '/' + connect_name
        elif device_type == 'Windows':
            devices = "Windows:///"
            if hwnd is not None:
                devices = devices + hwnd
            elif title is not None:
                devices = devices + '?title_re=' + title + '.*'
        auto_setup(__file__, logdir=False, devices=[devices])

    @staticmethod
    def snapshot(img_path: str = ''):
        """
        设备截图函数，打印或不打印图片到指定路径

        :param img_path: 图片路径及名称（打印）
        :return: ndarray
        """
        if img_path:
            screen = snapshot(img_path, quality=99)
        else:
            screen = G.DEVICE.snapshot(quality=99)
        return screen

    @staticmethod
    def loop_find(template: Template, timeout=ST.FIND_TIMEOUT, threshold=None, interval=0.5, intervalfunc=None,
                  random_area=0):
        """
        判断模板图片在设备上是否存在，如果存在返回坐标

        :param template: 图片类
        :param timeout:  超时时间
        :param threshold: 相似度
        :param interval:  识别间隔
        :param intervalfunc:
        :param random_area: 随机区域，默认0，取中心点，大于零小于等于1则区域内随机
        :return:
        """
        G.LOGGING.info("Try finding: %s", template)
        start_time = time.time()
        while True:
            screen = G.DEVICE.snapshot(filename=None, quality=ST.SNAPSHOT_QUALITY)

            if screen is None:
                G.LOGGING.warning("Screen is None, may be locked")
            else:
                if threshold:
                    template.threshold = threshold
                if 0 < random_area <= 1:
                    # 获取区域信息
                    match_rectangle = template._cv_match(screen).get("rectangle")
                    # 区域坐标
                    x1, y1 = match_rectangle[0]
                    x2, y2 = match_rectangle[1]
                    x3, y3 = match_rectangle[2]
                    x4, y4 = match_rectangle[3]
                    # 计算长方形的宽度和高度
                    width = max(x1, x2, x3, x4) - min(x1, x2, x3, x4)
                    height = max(y1, y2, y3, y4) - min(y1, y2, y3, y4)
                    # 处理随机区域
                    if random_area < 1:
                        x1, y1 = x1 + random_area * width / 2, y1 + random_area * height / 2
                        x2, y2 = x2 + random_area * width / 2, y2 - random_area * height / 2
                        x3, y3 = x3 - random_area * width / 2, y3 - random_area * height / 2
                        x4, y4 = x4 - random_area * width / 2, y4 + random_area * height / 2
                    random_x = int(random.uniform(min(x1, x2, x3, x4), max(x1, x2, x3, x4)))
                    random_y = int(random.uniform(min(y1, y2, y3, y4), max(y1, y2, y3, y4)))
                    match_pos = (random_x, random_y)
                else:
                    # 其他情况，取中心点
                    match_pos = template.match_in(screen)
                if match_pos:
                    try_log_screen(screen)
                    return match_pos

            if intervalfunc is not None:
                intervalfunc()

            # 超时则raise，未超时则进行下次循环:
            if (time.time() - start_time) > timeout:
                try_log_screen(screen)
                raise TargetNotFoundError('Picture %s not found in screen' % template)
            else:
                time.sleep(interval)

    @staticmethod
    def exists(template: Template, cvstrategy: [] = Settings.CVSTRATEGY, timeout: float = Settings.FIND_TIMEOUT_TMP,
               random_area=0):
        """
        判断模板图片在设备上是否存在，如果存在返回坐标

        :param template: 图片类
        :param cvstrategy: 图像识别算法
        :param timeout: 超时时间
        :param random_area: 随机区域，默认0，取中心点，大于零小于等于1则区域内随机
        :return: bool
        """
        Settings.CVSTRATEGY = cvstrategy
        Settings.FIND_TIMEOUT_TMP = timeout
        try:
            pos = BasicAirtest.loop_find(template, timeout=ST.FIND_TIMEOUT_TMP, random_area=random_area)
        except TargetNotFoundError:
            return False
        else:
            return pos

    @staticmethod
    def touch(template: Template, cvstrategy: [] = Settings.CVSTRATEGY, timeout: float = Settings.FIND_TIMEOUT_TMP,
              times: int = 1, **kwargs):
        """
        判断模板图片在设备上是否存在，如果存在点击
        :param times: 点击次数
        :param cvstrategy: 图像识别算法
        :param template: 图片类
        :param timeout: 超时时间
        :return: bool

        """
        Settings.CVSTRATEGY = cvstrategy
        Settings.FIND_TIMEOUT = timeout
        pos = loop_find(template, timeout=ST.FIND_TIMEOUT)
        for _ in range(times):
            pos = G.DEVICE.touch(pos, **kwargs) or pos
            time.sleep(0.05)
        delay_after_operation()
        return pos

    @staticmethod
    def touch_coordinate(pos: (), times: int = 1, **kwargs):
        """
        点击坐标
        :param pos: 坐标信息（x,y）
        :param times: 点击次数
        :return: bool
        """
        for _ in range(times):
            pos = G.DEVICE.touch(pos, **kwargs) or pos
            time.sleep(0.05)
        delay_after_operation()
        return pos

    @staticmethod
    def adb_stop_app(package: str):
        """
        停止APP

        :param package: app的包名
        :return: 无
        """
        stop_app(package=package)

    @staticmethod
    def adb_start_app(package: str):
        """
        启动APP

        :param package: app的包名
        :return: 无
        """
        start_app(package=package)

    @staticmethod
    def adb_restart_app(package: str):
        """
        重启APP

        :param package: app的包名
        :return: 无
        """
        BasicAirtest.adb_stop_app(package)
        time.sleep(2)
        BasicAirtest.adb_start_app(package)
        time.sleep(2)

    @staticmethod
    def swipe(v1: [], v2: [], duration:float=DURATION):
        """
        滑动
        :param duration: 间隔
        :param v1: 坐标1
        :param v2: 坐标2
        :return: bool
        """
        if swipe(v1, v2, duration=duration):
            return True
        else:
            return False

    @staticmethod
    def crop_image(x1, y1, x2, y2):
        """
        局部截图

        :param x1: x1
        :param y1: y1
        :param x2: x2
        :param y2: y2
        :return: ndarray
        """
        screen = G.DEVICE.snapshot()
        # 局部截图
        local_screen = aircv.crop_image(screen, (x1, y1, x2, y2))
        return local_screen

    @staticmethod
    def resolution_ratio():
        """
        获取当前设备分辨率

        :return: Tuple
        """
        if G.DEVICE.display_info['orientation'] in [1, 3]:
            height = G.DEVICE.display_info['width']
            width = G.DEVICE.display_info['height']
        else:
            height = G.DEVICE.display_info['height']
            width = G.DEVICE.display_info['width']
        return width, height

    @staticmethod
    def cvt_color(screen):
        """
        颜色空间转换 BGR->RGB

        :param screen: 图片
        :return: ndarray
        """
        return cv2_2_pil(screen)

    @staticmethod
    def find_all(template: Template, cvstrategy: [] = Settings.CVSTRATEGY, timeout: float = Settings.FIND_TIMEOUT):
        """
        在设备屏幕上查找所有出现的目标并返回它们的坐标

        :param template: template对象
        :param timeout: 超时时间
        :param cvstrategy:  图像识别算法
        :return: [{'result': (x, y),'rectangle': ( (left_top, left_bottom, right_bottom, right_top) ),'confidence': 0.9},...]
        """
        Settings.CVSTRATEGY = cvstrategy
        Settings.FIND_TIMEOUT = timeout
        result = find_all(template)
        return result

    @staticmethod
    def cv_match(template: Template, screen, cvstrategy: [] = Settings.CVSTRATEGY):
        """
        图片中查找图片信息

        :param template: 图片1 template格式
        :param screen: 图片2 ndarray格式
        :param cvstrategy: 图像识别算法
        :return: [{'result': (x, y),'rectangle': ( (left_top, left_bottom, right_bottom, right_top) ),'confidence': 0.9},...]
        """
        Settings.CVSTRATEGY = cvstrategy
        return template._cv_match(screen)

    @staticmethod
    def match_in(template: Template, screen, cvstrategy: [] = Settings.CVSTRATEGY,
                 timeout: float = Settings.FIND_TIMEOUT_TMP):
        """
        判断图片是否存在图片并返回坐标（x,y）

        :param screen:   局部截图
        :param template: 图片类
        :param cvstrategy: 图像识别算法
        :param timeout: 超时时间
        :return: (x,y)
        """
        Settings.CVSTRATEGY = cvstrategy
        Settings.FIND_TIMEOUT_TMP = timeout
        return template.match_in(screen)

    @staticmethod
    def text(word: str):
        """
        输入文字

        :param word: 文字内容
        :return: None
        """
        text1 = "input text '" + word + "'"
        shell(text1)

    @staticmethod
    def get_cap_method(serialno):
        """
        获取截图方法

        :param serialno: 安卓设备序列号
        :return: str
        """
        dev = Android(serialno=serialno)
        screen_proxy = ScreenProxy.auto_setup(dev.adb, rotation_watcher=dev.rotation_watcher)
        all_methods = screen_proxy.SCREEN_METHODS
        methods_class = screen_proxy.screen_method
        for index, (key, value) in enumerate(all_methods.items(), start=1):
            if isinstance(methods_class, value):
                return key
        return None

    @staticmethod
    def check_fast_method(serialno):
        """
        检查最快的、可用的截图方法

        :param serialno: 安卓设备序列号
        :return: str
        """
        dev = Android(serialno=serialno)
        screen_proxy = ScreenProxy.auto_setup(dev.adb, rotation_watcher=dev.rotation_watcher)
        all_methods = screen_proxy.SCREEN_METHODS
        # 从self.SCREEN_METHODS中，逆序取出可用的方法
        best_method = None
        best_time = None
        for name, screen_class in reversed(all_methods.items()):
            screen = screen_class(dev.adb, rotation_watcher=dev.rotation_watcher)
            now1 = time.time()
            result = screen_proxy.check_frame(screen)
            now2 = time.time()
            best_time1 = now2 - now1
            if result:
                if best_time1:
                    if best_time is None:
                        best_time = best_time1
                        best_method = name
                    elif best_time1 < best_time:
                        best_time = best_time1
                        best_method = name
        return best_method



import os
import sys
import codecs
from setuptools import setup, find_packages


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, str(rel_path)), 'r', encoding="utf-8") as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            # __version__ = "1.x.x"
            deli = '"' if '"' in line else "'"
            return line.split(deli)[1]
    else:
        raise RuntimeError("Unable to find version string.")


def is_docker():
    if os.path.exists('/proc/self/cgroup'):
        with open('/proc/self/cgroup', 'rt') as ifh:
            return 'docker' in ifh.read()
    return False


def parse_requirements(filename):
    """ load requirements from a pip requirements file. (replacing from pip.req import parse_requirements)"""
    linter = (line.strip() for line in open(filename))
    reqs = [line for line in linter if line and not line.startswith("#")]
    if sys.platform == "win32":
        reqs.append('pywin32')
    return reqs


setup(
    name='autopc',
    version=get_version("autopc/utils/utils_version.py"),
    author='orcakill',
    author_email='orcakill@dingtalk.com',
    description='An image recognition framework running on a computer',
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/orcakill/autopc',
    license='Apache License 2.0',
    keywords=['automation', 'opencv-python', 'ocr'],
    packages=find_packages(exclude=['test', 'test.*', 'dist', 'dist.*']),
    include_package_data=True,
    install_requires=parse_requirements('requirements.txt'),
    extras_require={
        'test': [
            'nose',
        ],
        'docs': [
            'sphinx',
            'recommonmark',
            'sphinx_rtd_theme',
            'mock',
        ]},
    classifiers=[
        'Programming Language :: Python :: 3.9',
    ],
)



# @Time: 2024年12月30日 00:38
# @Author: orcakill
# @File: utils_logger.py
# @Description: 日志文件
import os
import sys
import warnings

from loguru import logger

logger.remove()  # 移除默认的日志记录器


class MyLogger:
    def __init__(self):
        self.logger = logger
        self.logger.remove()

    def configure_logger(self, log_file_path):
        # 添加控制台输出的格式
        self.logger.add(
            sys.stdout,
            level='DEBUG',
            format="<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> | "
                   "{process.name} | "
                   "{thread.name} | "
                   "<cyan>{module}</cyan>.<cyan>{function}</cyan>"
                   ":<cyan>{line}</cyan> | "
                   "<level>{level}</level>: "
                   "<level>{message}</level>",
        )

    def get_logger(self):
        return self.logger


def loguru_showwarning(message, category):
    logger.warning(f"{category.__name__}: {message}")


# 屏蔽第三方包日志
logger.disable("airtest")

# 创建MyLogger实例
my_logger = MyLogger().get_logger()



# @Time: 2024年10月21日 14:21
# @Author: orcakill
# @File: basic_adb.py
# @Description: adb相关API,执行adb命令
import subprocess


class BasicADB:

    @staticmethod
    def get_adb_resolution(device_address):
        """
        根据设备信息获取Android设备分辨率

        :param device_address:
        :return:
        """
        command = 'adb -s ' + device_address + ' shell wm size'
        process = subprocess.Popen(command.split(), stdout=subprocess.PIPE)
        output, error = process.communicate()
        resolution_tuple = output.decode().strip().split(' ')[-1]
        resolution_tuple = tuple(map(int, resolution_tuple.split('x')))
        if resolution_tuple:
            print(resolution_tuple)
            return resolution_tuple
